/* artistDialog.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "artistDialog.h"
#include "artistParser.h"
#include "dataCell.h"
#include "artist.h"
#include "artistLookup.h"
#include "string_utils.h"
#include "playlistDialog.h"

ArtistDialog::
ArtistDialog(const Dataset &data, int form_id) :
  PalomaDialog(data),
  _form_id(form_id),
  _menu("<artist>"),
  _names(2, 5),
  _last_name_label("Last name:"),
  _first_name_label("First name:"),
  _name_suffix_label("Jr., Sr., etc:"),
  _group_prefix_label("A, An, The:"),
  _group_name_label("Group name:"),
  _as_group_button("Interpret as group"),
  _as_individual_button("Interpret as individual"),
  _ok_button("OK"),
  _new_button("New"),
  _lookup_button("Look up"),
  _cancel_button("Cancel")
{
  _changing = false;
  _lookup = NULL;
  
  set_entries();
  layout_window();
  setup();
}

ArtistDialog::
~ArtistDialog() {
  delete_lookup();
}

void ArtistDialog::
record_db() {
  int artist_id = _data[TI_artists].record_db();
  _data[TI_artists_extension].record_db(artist_id);
}

void ArtistDialog::
popup_lookup() {
  delete_lookup();
  _lookup = new ArtistLookup(_data, false);
  connect_to_method(_lookup->deleted, this, &lookup_deleted);
  connect_to_method(_lookup->accepted, this, &lookup_accepted);
}

void ArtistDialog::
delete_lookup() {
  if (_lookup != NULL) {
    delete _lookup;
    _lookup = NULL;
  }
}

void ArtistDialog::
menu_cds() {
  PlaylistDialog::make_dialog
    (_data, -1, 
     "SELECT cds.cd_id, cds.title"
     " FROM cds"
     " WHERE cds.performer=" + DataCell(_data[TI_artists].get_primary_key()) +
     " OR cds.writer=" + DataCell(_data[TI_artists].get_primary_key()));
}

void ArtistDialog::
menu_songs() {
  PlaylistDialog::make_dialog
    (_data, -1, 
     "SELECT songs.song_id, songs.name"
     " FROM songs"
     " WHERE songs.performer=" + DataCell(_data[TI_artists].get_primary_key()) +
     " OR songs.writer=" + DataCell(_data[TI_artists].get_primary_key()));
}

void ArtistDialog::
set_entries() {
  // We have this internal recursion-defeating flag.  We set this to
  // indicate we're setting the entries directly, so that the
  // callbacks attached to them know not to try to do anything.
  _changing = true;

  _last_name_entry.set_text(_data[TI_artists][AFI_last_name].get_string());
  _first_name_entry.set_text(_data[TI_artists][AFI_first_name].get_string());
  _name_suffix_entry.set_text(_data[TI_artists][AFI_name_suffix].get_string());
  _group_prefix_entry.set_text(_data[TI_artists][AFI_group_prefix].get_string());
  _group_name_entry.set_text(_data[TI_artists][AFI_group_name].get_string());

  _formatted_name_label.set_text(get_formatted_artist_name(_data[TI_artists]));
  _sorting_name_label.set_text(get_sorting_artist_name(_data[TI_artists]));

  _changing = false;
}

void ArtistDialog::
read_entries() {
  string last_name = trim_blanks(_last_name_entry.get_text());
  string first_name = trim_blanks(_first_name_entry.get_text());
  string name_suffix = trim_blanks(_name_suffix_entry.get_text());
  string group_prefix = trim_blanks(_group_prefix_entry.get_text());
  string group_name = trim_blanks(_group_name_entry.get_text());

  _data[TI_artists][AFI_last_name] = DataCell(last_name);
  _data[TI_artists][AFI_first_name] = DataCell(first_name);
  _data[TI_artists][AFI_name_suffix] = DataCell(name_suffix, !name_suffix.empty());
  _data[TI_artists][AFI_group_prefix] = DataCell(group_prefix, !group_prefix.empty());
  _data[TI_artists][AFI_group_name] = DataCell(group_name);

  _formatted_name_label.set_text(get_formatted_artist_name(_data[TI_artists]));
  _sorting_name_label.set_text(get_sorting_artist_name(_data[TI_artists]));
}


void ArtistDialog::
anything_changed() {
  if (!_changing) {
    read_entries();
  }
}

void ArtistDialog::
as_group_button_clicked() {
  read_entries();
  ArtistParser p(_formatted_name_label.get_text(), ArtistParser::H_group);

  _data[TI_artists][AFI_last_name] = DataCell(p._last_name);
  _data[TI_artists][AFI_first_name] = DataCell(p._first_name);
  _data[TI_artists][AFI_name_suffix] = DataCell(p._name_suffix, !p._name_suffix.empty());
  _data[TI_artists][AFI_group_prefix] = DataCell(p._group_prefix, !p._group_prefix.empty());
  _data[TI_artists][AFI_group_name] = DataCell(p._group_name);

  set_entries();
}

void ArtistDialog::
as_individual_button_clicked() {
  read_entries();
  ArtistParser p(_formatted_name_label.get_text(), ArtistParser::H_individual);

  _data[TI_artists][AFI_last_name] = DataCell(p._last_name);
  _data[TI_artists][AFI_first_name] = DataCell(p._first_name);
  _data[TI_artists][AFI_name_suffix] = DataCell(p._name_suffix, !p._name_suffix.empty());
  _data[TI_artists][AFI_group_prefix] = DataCell(p._group_prefix, !p._group_prefix.empty());
  _data[TI_artists][AFI_group_name] = DataCell(p._group_name);

  set_entries();
}

void ArtistDialog::
ok_button_clicked() {
  read_entries();
  accepted();

  delete_self();
}

void ArtistDialog::
new_button_clicked() {
  _data[TI_artists] = TableDataProxy(TI_artists);
  set_entries();
}

void ArtistDialog::
lookup_button_clicked() {
  popup_lookup();
}

void ArtistDialog::
cancel_button_clicked() {
  delete_self();
}

// Called on return from the ArtistLookup, created via popup_lookup().
void ArtistDialog::
lookup_accepted() {
  assert(_lookup != NULL);
  _data[TI_artists] = _lookup->get_artist();
  set_entries();
}

void ArtistDialog::
lookup_deleted() {
  _lookup = NULL;
}

void ArtistDialog::
layout_window() {
  set_default_size(400, 0);
  set_title("Define artist");

  // The menu bar.
  _menubar = _menu.get_menubar_widget("");
  assert(_menubar.get_object() != NULL);

  _menu.create_item("/File/CD's by this artist", "", "",
		    ItemFactoryConnector<ArtistDialog, int>(this, &menu_cds));
  _menu.create_item("/File/Songs by this artist", "", "",
		    ItemFactoryConnector<ArtistDialog, int>(this, &menu_songs));
  _menu.create_item("/File/sep", "", "<Separator>", 0);
  _menu.create_item("/File/Save", "", "",
		    ItemFactoryConnector<ArtistDialog, int>(this, &record_db));
  _menu.create_item("/File/Close", "", "",
		    ItemFactoryConnector<ArtistDialog, int>(this, &ok_button_clicked));

  vbox_.pack_start(*_menubar.get_object(), false, false);
  _menubar.get_object()->show();

  // The box for the main set of widgets.
  vbox_.pack_start(_box1);
  _box1.show();
  _box1.set_border_width(8);

  // A table to hold all the name fields.
  _box1.pack_start(_names, false, false, 0);
  _names.show();

  // Last name.
  _last_name_label.set_alignment(1, 0);
  _names.attach(_last_name_label, 0, 1, 0, 1, GTK_FILL, GTK_FILL);
  _names.attach(_last_name_entry, 1, 2, 0, 1);
  _last_name_label.show();
  _last_name_entry.show();
  connect_to_method(_last_name_entry.changed, this, &anything_changed);

  // First name.
  _first_name_label.set_alignment(1, 0);
  _names.attach(_first_name_label, 0, 1, 1, 2, GTK_FILL, GTK_FILL);
  _names.attach(_first_name_entry, 1, 2, 1, 2);
  _first_name_label.show();
  _first_name_entry.show();
  connect_to_method(_first_name_entry.changed, this, &anything_changed);

  // Suffix.
  _name_suffix_label.set_alignment(1, 0);
  _names.attach(_name_suffix_label, 0, 1, 2, 3, GTK_FILL, GTK_FILL);
  _names.attach(_name_suffix_entry, 1, 2, 2, 3);
  _name_suffix_label.show();
  _name_suffix_entry.show();
  connect_to_method(_name_suffix_entry.changed, this, &anything_changed);

  // Prefix.
  _group_prefix_label.set_alignment(1, 0);
  _names.attach(_group_prefix_label, 0, 1, 3, 4, GTK_FILL, GTK_FILL);
  _names.attach(_group_prefix_entry, 1, 2, 3, 4);
  _group_prefix_label.show();
  _group_prefix_entry.show();
  connect_to_method(_group_prefix_entry.changed, this, &anything_changed);

  // Group name.
  _group_name_label.set_alignment(1, 0);
  _names.attach(_group_name_label, 0, 1, 4, 5, GTK_FILL, GTK_FILL);
  _names.attach(_group_name_entry, 1, 2, 4, 5);
  _group_name_label.show();
  _group_name_entry.show();
  connect_to_method(_group_name_entry.changed, this, &anything_changed);

  // The names to show how it will be formatted.
  vbox_.pack_start(_formatted_name_label, true, false, 0);
  _formatted_name_label.show();
  /*
  vbox_.pack_start(_sorting_name_label, true, false, 0);
  _sorting_name_label.show();
  */

  // The box for the "as group" and "as individual" buttons.
  vbox_.pack_start(_as_box, true, true, 8);
  _as_box.show();

  // The "As Group" button.
  _as_group_button.set_usize(180, 24);
  _as_box.pack_start(_as_group_button, true, false, 0);
  _as_group_button.show();
  connect_to_method(_as_group_button.clicked, this, &as_group_button_clicked);

  // The "As Individual" button.
  _as_individual_button.set_usize(180, 24);
  _as_box.pack_start(_as_individual_button, true, false, 0);
  _as_individual_button.show();
  connect_to_method(_as_individual_button.clicked, this, &as_individual_button_clicked);

  // The OK button.
  _ok_button.set_usize(80, 36);
  action_area_.pack_start(_ok_button, true, false, 0);
  _ok_button.set_flags(GTK_CAN_DEFAULT);
  _ok_button.grab_default();
  _ok_button.show();
  connect_to_method(_ok_button.clicked, this, &ok_button_clicked);

  // The New button.
  _new_button.set_usize(80, 36);
  action_area_.pack_start(_new_button, true, false, 0);
  _new_button.set_flags(GTK_CAN_DEFAULT);
  _new_button.show();
  connect_to_method(_new_button.clicked, this, &new_button_clicked);

  // The Lookup button.
  _lookup_button.set_usize(80, 36);
  action_area_.pack_start(_lookup_button, true, false, 0);
  _lookup_button.set_flags(GTK_CAN_DEFAULT);
  _lookup_button.show();
  connect_to_method(_lookup_button.clicked, this, &lookup_button_clicked);
  
  // The Cancel button.
  _cancel_button.set_usize(80, 36);
  action_area_.pack_start(_cancel_button, true, false, 0);
  _cancel_button.set_flags(GTK_CAN_DEFAULT);
  _cancel_button.show();
  connect_to_method(_cancel_button.clicked, this, &cancel_button_clicked);
}
