/* artistLookup.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "artistLookup.h"
#include "artistDialog.h"
#include "database.h"
#include "dataCell.h"
#include "artist.h"
#include "artistMergeDialog.h"
#include "format.h"
#include "playlistDialog.h"
#include "infoMessage.h"

#include <algorithm>

static const int num_clist_titles = 1;
static const gchar *
clist_titles[num_clist_titles] = { "Artist" };

ArtistLookup::AcceptDialogData::
AcceptDialogData(ArtistDialog *dialog, int artist_index) :
  _dialog(dialog), _artist_index(artist_index) {
}

ArtistLookup::
ArtistLookup(const Dataset &data, bool edit_mode) :
  PalomaDialog(data),
  _edit_mode(edit_mode),
  _menu("<artist-lookup>"),
  _clist(num_clist_titles, clist_titles),
  _ok_button("OK"),
  _edit_button("Edit"),
  _cancel_button("Cancel")
{
  layout_window();
  setup();
}

ArtistLookup::
~ArtistLookup() {
}

// get_artist() returns the currently selected artist, if any.
TableDataProxy ArtistLookup::
get_artist() const {
  if (_clist.selbegin() != _clist.selend()) {
    int row = *(_clist.selbegin());
    return TableDataProxy(TI_artists, _artist_list[row]);
  }

  return TableDataProxy(TI_artists);
}

const ArtistLookup::ArtistList &ArtistLookup::
get_artist_list() const {
  return _artist_list;
}

void ArtistLookup::
popup_artist_dialog(int artist_index) {
  assert(artist_index >= 0 && artist_index < _artist_list.size());
  int artist_id = _artist_list[artist_index];

  Dataset new_data = _data;
  new_data[TI_artists] = TableDataProxy(TI_artists, artist_id);
  new_data[TI_artists_extension] = 
    TableDataProxy(TI_artists_extension, artist_id);

  ArtistDialog *dialog = new ArtistDialog(new_data, -1);
  connect_to_method(dialog->accepted, this, &artist_dialog_accepted, 
		    AcceptDialogData(dialog, artist_index));
}

void ArtistLookup::
menu_make_playlist() {
  // First, build up the list of artist id's we have selected.
  string artist_ids;
  for (Gtk_CList::seliterator si = _clist.selbegin();
       si != _clist.selend();
       ++si) {
    assert(*si >= 0 && *si < _artist_list.size());
    artist_ids += ", " + format_int(_artist_list[*si]);
  }

  if (artist_ids.empty()) {
    new InfoMessage
      ("You have no rows selected.  Use the mouse to select the artists\n"
       "you want to put in your playlist, and choose this option again.");
  } else {
    // Now issue a cheesy select to get all those artists.
    PlaylistDialog::make_dialog
      (_data, -1, 
       "SELECT artists.artist_id"
       " FROM artists"
       " WHERE artists.artist_id IN (" + artist_ids.substr(2) + ")");
  }
}  

void ArtistLookup::
menu_merge() {
  // Picking the merge option means to unify all the selected artists
  // into one.
  if (_clist.selbegin() != _clist.selend()) {
    // Is more than one artist selected?
    Gtk_CList::seliterator si = _clist.selbegin();
    int first_artist = _artist_list[*si];
    ++si;
    if (si != _clist.selend()) {
      // Build up the list of artist ID's selected.
      ArtistMergeDialog::MergeArtists artists;
      while (si != _clist.selend()) {
	artists.push_back(_artist_list[*si]);
	++si;
      }

      _clist.unselect_all();
      ArtistMergeDialog *dialog =
	new ArtistMergeDialog(_data, first_artist, artists);
      add_child_dialog(dialog);
      connect_to_method(dialog->accepted, this, &merge_dialog_accepted,
			dialog);
    }
  }
}

void ArtistLookup::
ok_button_clicked() {
  // In edit mode, the ok button brings up the selected CD if we have
  // exactly one row selected, and it brings up a playlist if we have
  // multiple rows selected.
  if (_edit_mode) {
    if (_clist.selbegin() != _clist.selend()) {
      Gtk_CList::seliterator si = _clist.selbegin();
      ++si;
      if (si == _clist.selend()) {
	// One row selected.
	int row = *(_clist.selbegin());
	popup_artist_dialog(row);
      } else {
	// Multiple rows selected.
	menu_make_playlist();
      }
    }
  }

  accepted();
  delete_self();
}

void ArtistLookup::
edit_button_clicked() {
  // Clicking the edit button means to edit the most-recently-selected
  // row, if any.
  if (_clist.selbegin() != _clist.selend()) {
    // In order to get the iterator just before the end, we must scan
    // the entire selection list from the beginning, since we don't
    // have selrbegin() and selrend().

    Gtk_CList::seliterator si, last_si;
    for (si = _clist.selbegin(); si != _clist.selend(); ++si) {
      last_si = si;
    }

    popup_artist_dialog(*last_si);
  }
}

void ArtistLookup::
cancel_button_clicked() {
  delete_self();
}

gint ArtistLookup::
clist_clicked(GdkEventButton *event) {
  // In edit mode, double-clicking a row means to pop up an edit
  // dialog for that row.  In non-edit mode, double-clicking a row
  // means to select the row and close the dialog.
  if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
    int row, column;
    _clist.get_selection_info(event->x, event->y, &row, &column);
    if (row >= 0 && row < _artist_list.size()) {
      if (_edit_mode) {
	// This cause_unselect_row() is supposed to undo the selecting
	// effects of the click, but it doesn't seem to have any
	// effect for some reason.  But we don't want to leave the row
	// selected.  As a compromise, we'll unselect it when we
	// accept the dialog.
	_clist.cause_unselect_row(row, 0);
	popup_artist_dialog(row);

      } else {
	_clist.cause_select_row(row, 0);
	accepted();
	delete_self();
      }
    }

    // Don't do any further processing of the double-click.
    return true;
  }

  return false;
}

void ArtistLookup::
artist_dialog_accepted(AcceptDialogData add) {
  add._dialog->record_db();
  const TableDataProxy &artist = add._dialog->get_data()[TI_artists];

  int t = add._artist_index;
  _clist.set_text(t, 0, get_sorting_artist_name(artist));
  _clist.cause_unselect_row(t, 0);
}

void ArtistLookup::
merge_dialog_accepted(ArtistMergeDialog *dialog) {
  // Ok, we've merged a bunch of artists together.  Remove all the
  // deleted artists from the list.
  const ArtistMergeDialog::MergeArtists &lose_artists =
    dialog->get_merge_artists();
  
  // We will simulatenously remove the items from the CList one at a
  // time while we build up an array in new_artists to replace
  // _artist_list.

  ArtistList new_artists;
  for (int t = 0; t < _artist_list.size(); t++) {
    if (find(lose_artists.begin(), lose_artists.end(), _artist_list[t]) !=
	lose_artists.end()) {
      
      // This artist is one that was just deleted.  Remove it from the
      // list.  The index number in the CList is not necessarily t,
      // because we might have deleted some elements before this one.
      // The index number is actually the length of the new_artists
      // list.

      _clist.remove_row(new_artists.size());

    } else {
      // This artist is perfectly fine.  Keep it.
      new_artists.push_back(_artist_list[t]);
    }
  }

  _artist_list.swap(new_artists);
}

void ArtistLookup::
layout_window() {
  set_default_size(450, 450);
  set_title("Artist Look-Up");

  // The menu bar.
  _menubar = _menu.get_menubar_widget("");
  assert(_menubar.get_object() != NULL);

  _menu.create_item("/File/Make a playlist", "", "",
		    ItemFactoryConnector<ArtistLookup, int>(this, &menu_make_playlist));
  _menu.create_item("/File/Edit", "", "",
		    ItemFactoryConnector<ArtistLookup, int>(this, &edit_button_clicked));
  _menu.create_item("/File/Accept", "", "",
		    ItemFactoryConnector<ArtistLookup, int>(this, &ok_button_clicked));
  _menu.create_item("/File/Merge", "", "",
		    ItemFactoryConnector<ArtistLookup, int>(this, &menu_merge));
  _menu.create_item("/File/Close", "", "",
		    ItemFactoryConnector<ArtistLookup, int>(this, &cancel_button_clicked));

  vbox_.pack_start(*_menubar.get_object(), false, false);
  _menubar.get_object()->show();

  // The box for the main set of widgets.
  vbox_.pack_start(_box1);
  _box1.show();
  _box1.set_border_width(8);

  // The list of artists.
  _artist_window.set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  _box1.pack_start(_artist_window, true, true, 0);
  _artist_window.show();

  _artist_window.add_with_viewport(_clist);

  _clist.column_titles_passive();
  _clist.set_selection_mode(GTK_SELECTION_MULTIPLE);
  _clist.show();

  const gchar *row_text[num_clist_titles] = {
    NULL,
  };

  QueryResult qr = music_db->sql_command
    ("SELECT artist_id,"
     " last_name, first_name, name_suffix,"
     " group_prefix, group_name,"
     " UPPER(" + music_db->concat(music_db->concat("last_name", "first_name"),
				  "group_name") + ") AS sort_artists"
     " FROM artists "
     " ORDER BY sort_artists");

  int num_rows = qr.get_num_rows();
  for (int t = 0; t < num_rows; t++) {
    int artist_id = qr.get_cell(t, 0).get_int();

    TableDataProxy artist(TI_artists);
    artist.set_db_value(AFI_last_name, qr.get_cell(t, 1));
    artist.set_db_value(AFI_first_name, qr.get_cell(t, 2));
    artist.set_db_value(AFI_name_suffix, qr.get_cell(t, 3));
    artist.set_db_value(AFI_group_prefix, qr.get_cell(t, 4));
    artist.set_db_value(AFI_group_name, qr.get_cell(t, 5));

    string artist_name = get_sorting_artist_name(artist);
    row_text[0] = artist_name.c_str();

    _clist.append(row_text);
    _artist_list.push_back(artist_id);
  }

  connect_to_method(_clist.button_press_event, this,
		    &clist_clicked);

  // The OK button.
  _ok_button.set_usize(80, 36);
  action_area_.pack_start(_ok_button, true, false, 0);
  _ok_button.set_flags(GTK_CAN_DEFAULT);
  _ok_button.grab_default();
  _ok_button.show();
  connect_to_method(_ok_button.clicked, this, &ok_button_clicked);
  
  // The Edit button.
  _edit_button.set_usize(80, 36);
  action_area_.pack_start(_edit_button, true, false, 0);
  _edit_button.set_flags(GTK_CAN_DEFAULT);
  _edit_button.show();
  connect_to_method(_edit_button.clicked, this, &edit_button_clicked);

  // The Cancel button.
  _cancel_button.set_usize(80, 36);
  action_area_.pack_start(_cancel_button, true, false, 0);
  _cancel_button.set_flags(GTK_CAN_DEFAULT);
  _cancel_button.show();
  connect_to_method(_cancel_button.clicked, this, &cancel_button_clicked);
}
