/* playlistControls.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "playlistControls.h"
#include "database.h"
#include "dataCell.h"
#include "song.h"
#include "artist.h"
#include "formDef.h"
#include "paloma.h"

#include "next.xpm"
#include "prev.xpm"

Gdk_Pixmap PlaylistControls::_next_gdk_pixmap;
Gdk_Bitmap PlaylistControls::_next_gdk_mask;
Gdk_Pixmap PlaylistControls::_prev_gdk_pixmap;
Gdk_Bitmap PlaylistControls::_prev_gdk_mask;
bool PlaylistControls::_got_pixmaps = false;

PlaylistControls::
PlaylistControls(const Dataset &data) : 
  _data(data),
  _player_controls(data)
{
  _cd_reader = NULL;

  make_pixmaps();
}

PlaylistControls::
~PlaylistControls() {
}

void PlaylistControls::
layout(const PlaylistProxy &playlist) {
  _playlist = playlist;

  _clist = _playlist.get_clist();
  assert(_clist != NULL);

  _cd_reader = _playlist.get_cd_reader();
  connect_to_method(_clist->select_row, this, &got_select_row);

  _next_button.add(_next_pixmap);
  _next_pixmap.show();
  _next_pixmap.set_build_insensitive(true);

  _prev_button.add(_prev_pixmap);
  _prev_pixmap.show();
  _prev_pixmap.set_build_insensitive(true);

  // The music controls.
  _prev_button.set_usize(36, 36);
  pack_start(_prev_button, false, true, 0);
  _prev_button.show();
  connect_to_method(_prev_button.clicked, this, &prev_button_clicked);

  pack_start(_player_controls, true, true, 0);
  _player_controls.show();
  connect_to_method(_player_controls.get_end_signal(), this, 
		    &song_player_stopped_at_end);

  _next_button.set_usize(36, 36);
  pack_start(_next_button, false, false, 0);
  _next_button.show();
  connect_to_method(_next_button.clicked, this, &next_button_clicked);
};


void PlaylistControls::
clear_player() {
  _cd_reader = NULL;
}

int PlaylistControls::
get_current_row() const {
  assert(_clist != NULL);
  return _clist->selbegin() == _clist->selend() ? -1 : *_clist->selbegin();
}

void PlaylistControls::
stop() {
  _player_controls.stop();
}

void PlaylistControls::
play() {
  _player_controls.play();
}

void PlaylistControls::
got_select_row(gint, gint, GdkEvent *) {
  // Rather than using the row number that comes in through the event,
  // here we'll use get_current_row() to determine the newly-selected
  // row.  We do this just in case, for some reason, the CList is in
  // some multiple-select mode, in which case the row we have just
  // selected is not necessarily the "current" row (defined to be the
  // first selected song).

  int row = get_current_row();
  assert(row >= 0 && row < _playlist.get_num_rows());
  if (_playlist.get_primary_table() == TI_songs &&
      _player_controls.get_current_song() != row) {
    const TableDataProxy &song = _playlist.get_table_data(row);
    _player_controls.set_song(song, _cd_reader, row);
  }
  
  _next_button.set_sensitive(row + 1 < _playlist.get_num_rows());
  _prev_button.set_sensitive(row - 1 >= 0);
}

void PlaylistControls::
song_player_stopped_at_end() {
  int current_row = get_current_row();
  if (current_row + 1 < _playlist.get_num_rows()) {
    _clist->cause_select_row(current_row + 1, 0);
  } else {
    // If we have no more songs to play, throw the "ended" event.
    ended();
  }
}

void PlaylistControls::
next_button_clicked() {
  int current_row = get_current_row();
  if (current_row + 1 < _playlist.get_num_rows()) {
    _clist->cause_select_row(current_row + 1, 0);
  }
}

void PlaylistControls::
prev_button_clicked() {
  int current_row = get_current_row();
  if (current_row - 1 >= 0) {
    _clist->cause_select_row(current_row - 1, 0);
  }
}

void PlaylistControls::
make_pixmaps() {
  Gdk_Window window = paloma->get_window();
  if (!_got_pixmaps) {
    _next_gdk_pixmap.create_from_xpm_d
      (window, _next_gdk_mask, NULL, next_xpm);
    _prev_gdk_pixmap.create_from_xpm_d
      (window, _prev_gdk_mask, NULL, prev_xpm);

    _got_pixmaps = true;
  }

  _next_pixmap.set(_next_gdk_pixmap, _next_gdk_mask);
  _prev_pixmap.set(_prev_gdk_pixmap, _prev_gdk_mask);
}
