/* sfClient.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef SFCLIENT_H
#define SFCLIENT_H

#include <gtk--.h>

// This class contacts the SFServer that should be running in the
// background and works with it to play sound files and CD tracks.

class SFClient : public Gtk_Signal_Base {
public:
  SFClient();
  ~SFClient();

  // is_alive() is true if we have a connection to the SFServer.
  bool is_alive() const;

  // Call reread_config() to advise the server to re-read its
  // configuration data because it may have changed.
  void reread_config();

  // Call set_filename() to prepare to play a sound file, or
  // set_cd_track() to prepare a track on the CD.  However, the
  // playback will not necessarily be ready immediately.  You cannot
  // call play() until the ready event has been thrown and is_ready()
  // returns true.
  void clear();
  void set_filename(const string &filename, int type, double length);
  void set_cd_track(const string &cdrom_device, int track_num, double length);

  // can_rstart() is true if the SFServer can start at a random
  // position within the sound file.  If this returns false the
  // position slider should be grayed out.
  bool can_rstart() const;

  // is_ready() is true whenever the sound file is ready to play.
  bool is_ready() const;

  // is_playing() is true whenever we are currently playing.
  bool is_playing() const;

  // play() begins the prepared sound file playing.  Start and stop
  // are the time in seconds, starting from zero, of the subset of the
  // sound file to play.
  void play(double start);
  void play(double start, double stop);

  // Call stop() to prematurely interrupt the playback.
  void stop();

  // rstart_state() is thrown whenever can_rstart() changes state, one way
  // or the other (the initial state is true).  The parameter is the
  // new state.
  Signal1<bool> rstart_state;

  // ready() is thrown at some point after set_filename() or
  // set_cd_track(), when is_ready() changes state from false to true
  // and the song is accepted and ready to play.
  Signal0 ready;

  // start_playing() is thrown whenever the server acknowledges a
  // play() request and is_playing() changes from false to true.
  Signal0 start_playing;

  // stop_playing() is thrown whenever the is_playing() changes from
  // true to false, for any reason.
  Signal0 stop_playing;

  // update_time() is thrown periodically while playing to update the
  // current position within the song.
  Signal1<double> update_time;

  // interrupt() will be thrown if the playback is stopped prematurely
  // due to another window starting a different sound file.
  Signal0 interrupt;

  // end() is thrown when playback reaches the end of the clip and
  // stops.
  Signal0 end;

  static int _port;
  static string _hostname;
  static bool _bad_hostname;
  static bool _no_server;

private:
  void set_rstart(bool flag);
  void set_playing(bool flag);
  void send_command(string command);
  int get_commands();

  void line_up_server_response(const string &input);
  void server_response(const string &line);

  int _server_socket;
  string _last_server_input;
  bool _is_alive;
  bool _can_rstart;
  bool _got_filename;
  bool _is_ready;
  bool _is_playing;
};

#endif
