/* songDialog.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "songDialog.h"
#include "song.h"
#include "cd.h"
#include "defaultForms.h"
#include "format.h"
#include "playlistDialog.h"

#include <stdio.h>

SongDialog::
SongDialog(const Dataset &data, int form_id, 
	   CDReader *cd_reader, int track_index) :
  PalomaDialog(data), _form_id(form_id), 
  _player_controls(data),
  _cd_reader(cd_reader), _track_index(track_index),
  _menu("<song>"),
  _name_label("Song name:"),
  _performer_label("Performer:"),
  _performer_entry(data),
  _writer_label("Writer / Composer:"),
  _writer_entry(data),
  _load_check_label("Load"),
  _ok_button("OK"),
  _cancel_button("Cancel")  
{
  DefaultForms::check_default(_form_id, TI_songs);
  _load = false;
  _name_entry.set_text(_data[TI_songs][SFI_name].get_string());
  _performer_entry.set_artist(_data[TI_songs][SFI_performer].get_foreign(TI_artists));
  _writer_entry.set_artist(_data[TI_songs][SFI_writer].get_foreign(TI_artists));

  DataCell loaded_time_dc = _data[TI_songs][SFI_loaded_time];

  if (loaded_time_dc.is_null()) {
    _loaded_time_label.set_text("Not yet loaded into computer.");

  } else {
    _loaded_time_label.set_text("Loaded into computer on " + 
				format_date(loaded_time_dc.get_time()) + ".");
    _load_check_label.set_text("Reload");
  }

  _player_controls.set_song(_data[TI_songs], _cd_reader, _track_index);

  if (_form_id >= 0) {
    _form.layout_form(_form_id, _data);
    _form.show();
  }

  layout_window();
  setup();
}

SongDialog::
~SongDialog() {
}

void SongDialog::
set_load(bool load) {
  _load = load;
  _load_check.set_active(_load);
}

bool SongDialog::
get_load() const {
  return _load;
}

void SongDialog::
record_db() {
  int song_id = _data[TI_songs].record_db();

  _data[TI_songs_extension].record_db(song_id);
}

void SongDialog::
menu_open_cds() {
  PlaylistDialog::make_dialog
    (_data, -1, 
     "SELECT cds.cd_id, cds.title"
     " FROM cds, cd_tracks"
     " WHERE cds.cd_id=cd_tracks.cd_id"
     " AND cd_tracks.song_id=" + DataCell(_data[TI_songs].get_primary_key()));
}  

void SongDialog::
menu_same_performer() {
  PlaylistDialog::make_dialog
    (_data, -1, 
     "SELECT songs.song_id, songs.name"
     " FROM songs"
     " WHERE songs.performer=" + DataCell(_performer_entry.get_artist()));
}  

void SongDialog::
ok_button_clicked() {
  _data[TI_songs][SFI_name] = DataCell(_name_entry.get_text());
  _data[TI_songs][SFI_performer] = DataCell(_performer_entry.get_artist());
  _data[TI_songs][SFI_writer] = DataCell(_writer_entry.get_artist());
  _load = _load_check.get_active();
  _form.update();

  // The SongDialog doesn't automatically record itself when the OK
  // button is clicked.  This is a special exception made by the
  // SongDialog; it will be up to its parent dialog to decide whether
  // it should be recorded or not.

  accepted();
  delete_self();
}

void SongDialog::
cancel_button_clicked() {
  delete_self();
}


void SongDialog::
layout_window() {
  set_default_size(400, 0);

  if (_cd_reader != NULL) {
    set_title("Edit track " + format_int(_track_index + 1) +
	      " of " + get_formatted_cd_name(_data[TI_cds]));
  } else {
    set_title("Edit song");
  }

  // The menu bar.
  _menubar = _menu.get_menubar_widget("");
  assert(_menubar.get_object() != NULL);

  _menu.create_item("/File/Open CD(s)", "", "",
		    ItemFactoryConnector<SongDialog, int>(this, &menu_open_cds));
  _menu.create_item("/File/Songs by this performer", "", "",
		    ItemFactoryConnector<SongDialog, int>(this, &menu_same_performer));
  _menu.create_item("/File/sep", "", "<Separator>", 0);
  _menu.create_item("/File/Save", "", "",
		    ItemFactoryConnector<SongDialog, int>(this, &record_db));
  _menu.create_item("/File/Close", "", "",
		    ItemFactoryConnector<SongDialog, int>(this, &ok_button_clicked));


  vbox_.pack_start(*_menubar.get_object(), false, false);
  _menubar.get_object()->show();

  // The box for the main set of widgets.
  vbox_.pack_start(_box1);
  _box1.show();
  _box1.set_border_width(8);

  // Song name, label and entry.
  _name_label.set_alignment(0, 0);
  _box1.pack_start(_name_label, false, false, 0);
  _box1.pack_start(_name_entry, false, false, 0);
  _name_label.show();
  _name_entry.show();

  // Performer, label and entry.
  _performer_label.set_alignment(0, 0);
  _box1.pack_start(_performer_label, false, false, 0);
  _box1.pack_start(_performer_entry, false, false, 0);
  _performer_label.show();
  _performer_entry.show();

  // Writer, label and entry.
  _writer_label.set_alignment(0, 0);
  _box1.pack_start(_writer_label, false, false, 0);
  _box1.pack_start(_writer_entry, false, false, 0);
  _writer_label.show();
  _writer_entry.show();

  // The loaded-time label.
  _box1.pack_start(_loaded_time_label, false, false, 0);
  _loaded_time_label.show();

  // The "load" check button.
  if (_cd_reader != NULL) {
    _box1.pack_start(_load_check, false, false, 0);
    _load_check.show();
    _load_check.add(_load_check_label);
    _load_check_label.show();
    _load_check_label.set_alignment(0, 0.5);
  }

  // The user's custom form, if defined.
  _box1.pack_start(_form, true, true, 0);

  // The stop/play/etc. controls.
  _box1.pack_start(_player_controls, false, false, 0);
  _player_controls.show();

  // The OK button.
  _ok_button.set_usize(80, 36);
  action_area_.pack_start(_ok_button, true, false, 0);
  _ok_button.set_flags(GTK_CAN_DEFAULT);
  _ok_button.grab_default();
  _ok_button.show();
  connect_to_method(_ok_button.clicked, this, &ok_button_clicked);
  
  // The Cancel button.
  _cancel_button.set_usize(80, 36);
  action_area_.pack_start(_cancel_button, true, false, 0);
  _cancel_button.set_flags(GTK_CAN_DEFAULT);
  _cancel_button.show();
  connect_to_method(_cancel_button.clicked, this, &cancel_button_clicked);
}
